@@ -8,7 +8,8 @@ from typing import Dict, List, Optional, Union
 
 import yaml
 
-from .base import ComplexValue, File, Header, Person, Value, ValueRange, ValueVector, orsodataclass
+from .. import dataclass
+from .base import AlternatingField, ComplexValue, File, Header, Person, Value, ValueRange, ValueVector
 from .model_language import SampleModel
 
 # typing stuff introduced in python 3.8
@@ -18,7 +19,7 @@ except ImportError:
     from .typing_backport import Literal
 
 
-@orsodataclass
+@dataclass
 class Experiment(Header):
     """
     A definition of the experiment performed.
@@ -43,7 +44,7 @@ class Experiment(Header):
     doi: Optional[str] = None
 
 
-@orsodataclass
+@dataclass
 class Sample(Header):
     """
     A description of the sample measured.
@@ -61,7 +62,9 @@ class Sample(Header):
     :param size: Sample size in x, y, z direction, where z is parallel to the surface normal
         and x is along the beam direction (important for footprint correction).
     :param environment: Name of the sample environment device(s).
-    :param sample_parameters: Dictionary of sample parameters.
+    :param sample_parameters: Dictionary of sample parameters. See Sample.PARAMETER_DEFAULT_KEYS
+        for some common names defined in the specification.
+    :param model: A model using the simple model description  (https://www.reflectometry.org/projects/simple_model)
     """
 
     name: str
@@ -70,11 +73,19 @@ class Sample(Header):
     description: Optional[str] = None
     size: Optional[ValueVector] = None
     environment: Optional[List[str]] = None
-    sample_parameters: Optional[Dict[str, Union[Value, ValueRange, ValueVector, ComplexValue]]] = field(
-        default=None, metadata={"description": "Using keys for parameters and Value* objects for values."}
-    )
+    sample_parameters: Optional[
+        Dict[str, Union[Value, ValueRange, ValueVector, ComplexValue, AlternatingField]]
+    ] = field(default=None, metadata={"description": "Using keys for parameters and Value* objects for values."})
     model: Optional[SampleModel] = None
 
+    PARAMETER_DEFAULT_KEYS = [
+        "temperature",
+        "magnetic_field",
+        "electric_potential",
+        "electric_current",
+        "electric_ac_field",
+    ]
+
 
 class Polarization(str, Enum):
     """
@@ -93,7 +104,7 @@ class Polarization(str, Enum):
     """
 
     unpolarized = "unpolarized"
-    # half polairzed states
+    # half polarized states
     po = "po"
     mo = "mo"
     op = "op"
@@ -118,13 +129,13 @@ class Polarization(str, Enum):
         return dumper.represent_str(output)
 
 
-@orsodataclass
+@dataclass
 class InstrumentSettings(Header):
     """
     Settings associated with the instrumentation.
 
     :param incident_angle: Angle (range) of incidence.
-    :param wavelength: Neutron/x-ray wavelenght (range).
+    :param wavelength: Neutron/x-ray wavelength (range).
     :param polarization: Radiation polarization as one of
         :code:`'unpolarized'`, :code:`'p'`, :code:`'m'`, :code:`'pp'`,
         :code:`'pm'`, :code:`'mp'`, :code:`'mm'`, or a
@@ -135,8 +146,8 @@ class InstrumentSettings(Header):
 
     incident_angle: Union[Value, ValueRange]
     wavelength: Union[Value, ValueRange]
-    polarization: Optional[Union[Polarization, ValueVector]] = field(
-        default=None,
+    polarization: Union[Polarization, ValueVector] = field(
+        default=Polarization.unpolarized,
         metadata={
             "description": "Polarization described as unpolarized/ po/ mo / op / om / pp / pm / mp / mm / vector"
         },
@@ -148,14 +159,14 @@ class InstrumentSettings(Header):
     __repr__ = Header._staggered_repr
 
 
-@orsodataclass
+@dataclass
 class Measurement(Header):
     """
     The measurement elements for the header.
 
     :param instrument_settings: Instrumentation details.
     :param data_files: Raw data files produced in the measurement.
-    :param references: Raw reference files used in the reduction.
+    :param additional_files: Raw reference files used in the reduction.
     :param scheme: Measurement scheme (one of :code:`'angle-dispersive'`,
         :code:`'energy-dispersive'`/:code:`'angle- and energy-dispersive'`).
     """
@@ -168,7 +179,7 @@ class Measurement(Header):
     __repr__ = Header._staggered_repr
 
 
-@orsodataclass
+@dataclass
 class DataSource(Header):
     """
     The data_source object definition.
